# BASE R plugins
#  by dpark@molbio.mgh.harvard.edu
# requires: "sma" R package (0.5.10) by Terry Speed's group @ Berkeley
#    http://www.stat.berkeley.edu/users/terry/zarray/Software/smacode.html
# requires: r-args.R and r-input.txt in expected formats
# plugin: pvalue
#	  created 3/18/03 by dpark
# plugin: foldratio
#	  created 3/28/03 by dpark
# plugin: avgA
#	  created 4/17/03 by dpark
# plugin: avgint
#	  created 5/06/03 by dpark
#

base.plug.version = "1.1.1"
base.plug.pval = "pvalue"
base.plug.foldratio = "foldratio"
base.plug.avgA = "avgA"
base.plug.avgint = "avgint"


## read in data
source("r-args.R")
read.baseplug<-function(fname="r-input.txt", ...)
{
    x <- read.table(fname, sep="\t", header=T, skip=0, as.is=T, ...)
    rownames(x) <- x$reporterId
    nhybs <- (dim(x)[2] - 1)/base.channels
    int1<-x[(0:(nhybs-1))*base.channels+2]
    if(base.channels < 2) {
       out<-list(int1=int1)
    } else {
        int2<-x[(0:(nhybs-1))*base.channels+3]
        RG<-list(R=int1, Rb=NULL, G=int2, Gb=NULL)
	library(sma)
        MA<-stat.ma(RG, norm="n", pout=F)     # "n" = none
        rownames(MA$M) <- rownames(x)
        rownames(MA$A) <- rownames(x)
	MA$A <- MA$A / log2(10) 
        if(base.channels < 3) {
            out<-c(MA, list(int1=int1, int2=int2))
        } else {
            int3<-x[(0:(nhybs-1))*base.channels+4]
            out<-c(MA, list(int1=int1, int2=int2, int3=int3))
	}
    }
    return(out)
}
base.dat <- read.baseplug()


## calculate data
if(base.channels >= 2) {
  if (base.arg.amethod == "mean") {
    M <- apply(base.dat$M, 1, mean, na.rm=T)
    A <- apply(base.dat$A, 1, mean, na.rm=T)
  } else if (base.arg.amethod == "median") {
    M <- apply(base.dat$M, 1, median, na.rm=T)
    A <- apply(base.dat$A, 1, median, na.rm=T)
  } else if (base.arg.amethod == "meanmid") {  ## mean of middle N-2
    len <- dim(base.dat$M)[2]
    mid <- 2:(len - 1)
    if(len == 1)  mid <- 1
    if(len == 2)  mid <- 1:2
    mean.middle <- function (...)
    {
       mean(sort(...)[mid], na.rm=T)
    }
    M <- apply(base.dat$M, 1, mean.middle)
    A <- apply(base.dat$A, 1, mean.middle)
  }
}
if(base.plugin == base.plug.pval) {

  if (base.arg.pmethod == "ttest") {
    pval.calc<-function(...) {
      if(sum(!is.na(...)) < 2) { return(NA) }
      t.test(...)$p.value
    }
  } else if (base.arg.pmethod == "wilcox") {
    pval.calc<-function(...) {
      if(sum(!is.na(...)) < 2) { return(NA) }
      wilcox.test(...)$p.value
    }
  }
  out <- apply(base.dat$M, 1, pval.calc)
  selector <- is.finite(out) & is.finite(M) & is.finite(A)
  out <- out[selector]
  M <- M[selector]
  A <- A[selector]

} else if(base.plugin == base.plug.foldratio) {

  selector <- is.finite(M) & is.finite(A)
  M <- M[selector]
  A <- A[selector]
  dat.log2 <- M
  out <- 2^dat.log2

} else if(base.plugin == base.plug.avgA) {

  selector <- is.finite(M) & is.finite(A)
  M <- M[selector]
  A <- A[selector]
  out <- A

} else if(base.plugin == base.plug.avgint) {

  if(base.arg.ch == 1) dat.in <- log2(base.dat$int1)
  if(base.arg.ch == 2) dat.in <- log2(base.dat$int2)
  if(base.arg.ch == 3) dat.in <- log2(base.dat$int3)
  if (base.arg.amethod == "mean") {
    dat.log2 <- apply(dat.in, 1, mean, na.rm=T)
  } else if (base.arg.amethod == "median") {
    dat.log2 <- apply(dat.in, 1, median, na.rm=T)
  } else if (base.arg.amethod == "meanmid") {  ## mean of middle N-2
    len <- dim(dat.in)[2]
    mid <- 2:(len - 1)
    if(len == 1)  mid <- 1
    if(len == 2)  mid <- 1:2
    mean.middle <- function (...)
    {
       mean(sort(...)[mid], na.rm=T)
    }
    dat.log2 <- apply(dat.in, 1, mean.middle)
  }

  selector <- is.finite(dat.log2) & is.finite(M) & is.finite(A)
  dat.log2 <- dat.log2[selector]
  M <- M[selector]
  A <- A[selector]
  out <- 2^dat.log2

}


## calculate summary stats
if((base.plugin == base.plug.foldratio) || (base.plugin == base.plug.avgint)) {
  mu <- mean(dat.log2)
  sd <- sqrt(var(dat.log2))
  sd.cutoffs <- 2^(mu + (-2:2 * sd))
} else {
  mu <- mean(out)
  sd <- sqrt(var(out))
  sd.cutoffs <- mu + (-2:2 * sd)
}
names(sd.cutoffs) <- c("mean - 2SD", "mean - SD", "mean",
  "mean + SD", "mean + 2SD")
qt.cutoffs <- quantile(out, 0:10/10)
names(qt.cutoffs) <- c("min", "10%", "20%", "30%", "40%", "median",
  "60%", "70%", "80%", "90%", "max")


## write summary stats to r-notes
if(base.make.summary) {
  write("Mean/Stddev statistics:", "r-notes.txt")
  write.table(sd.cutoffs, "r-notes.txt", append=T, sep="\t",
    row.names=T, col.names=F, quote=F)
  write("\nMedian/Quantile statistics:", "r-notes.txt", append=T)
  write.table(qt.cutoffs, "r-notes.txt", append=T, sep="\t",
    row.names=T, col.names=F, quote=F)
}


## write gene list
if(base.make.replist) {
  write.table(out, "r-output.txt", sep="\t", row.names=T, col.names=F, quote=F)
}


## generate plots
if(base.make.graph) {
  if(base.plugin == base.plug.pval) {
    bitmap("gene-pvalues.png", type="png256",
      height=base.fig.height, width=base.fig.width)
    l10p <- log10(out);         l10p <- l10p[is.finite(l10p)]
    hist(l10p, breaks="fd", freq=T,
      xlab="log_10 p-values (vert lines denote 10% quantiles)",
      main="Histogram of p-values across all Reporters")
    l10qt <- log10(qt.cutoffs); l10qt <- l10qt[is.finite(l10qt)]
    abline(v=l10qt, col="gray")
    dev.off()

    bitmap("gene-volcano.png", type="png256",
      height=base.fig.height, width=base.fig.width)
    l10p <- -log10(out)
    l2rat <- M[is.finite(l10p)]
    l10p <- l10p[is.finite(l10p)]
    plot(l2rat, l10p, type="p", col="red", pch=".", #pch=20,
      xlab="M = log2 ratio", ylab="-log10 pvalue",
      main="Volcano plot across all Reporters")
    dev.off()

  } else if(base.plugin == base.plug.foldratio) {
    bitmap("gene-foldchanges.png", type="png256",
      height=base.fig.height, width=base.fig.width)
    hist(dat.log2, breaks="fd", freq=T,
      axes=F,
      xlab="log_2 Fold Change (vert lines = 2, 1, and 0 std devs from mean)",
      main="Histogram of Fold Changes across all Reporters")
    xl <- floor(  1.5 * (log2(sd.cutoffs[1])-mu) + mu)
    xh <- ceiling(1.5 * (log2(sd.cutoffs[5])-mu) + mu)
    xlim <- xl:xh
    axis(1, at=xlim)
    axis(2)
    abline(v=log2(sd.cutoffs), col="gray")
    dev.off()

  } else if(base.plugin == base.plug.avgA) {
    bitmap("gene-avgA.png", type="png256",
      height=base.fig.height, width=base.fig.width)
    hist(out, breaks="fd", freq=T, axes=T,
      xlab="avg A (vert lines denote 10% quantiles)",
      main="Histogram of A across all Reporters")
    abline(v=qt.cutoffs, col="gray")
    dev.off()

  } else if(base.plugin == base.plug.avgint) {
    bitmap("gene-avgint.png", type="png256",
      height=base.fig.height, width=base.fig.width)
    hist(log10(out), breaks="fd", freq=T, axes=T,
      xlab="log_10 avg intensities (vert lines = 2, 1, and 0 std devs from mean)",
    main="Histogram of avg intensities across all Reporters")
    abline(v=log10(qt.cutoffs), col="gray")
    dev.off()

  }
}
if((base.make.maplot) && (base.channels >= 2)) {
  bitmap("gene-maplot.png", type="png256",
    height=base.fig.height, width=base.fig.width)
  maxm <- ceiling(max(abs(M))*1.05)
  plot(A, M, type="p", col="red", pch=".", #pch=20, 
    xlab="A", ylab="M", ylim=c(-maxm,maxm),
    main="M-A plot across all Reporters")
  abline(h=c(0), col="black")
  dev.off()
}


## end of script
